@@ -808,4 +808,6 @@ static const MVMREPROps this_repr = {
     "CUnion", /* name */
     MVM_REPR_ID_MVMCUnion,
     0, /* refs_frames */
+    NULL, /* unmanaged_size */
+    NULL, /* describe_refs */
 };
