@@ -204,12 +204,15 @@ static const MVMREPROps this_repr = {
         at_pos,
         MVM_REPR_DEFAULT_BIND_POS,
         MVM_REPR_DEFAULT_SET_ELEMS,
-        MVM_REPR_DEFAULT_EXISTS_POS,
         push,
         MVM_REPR_DEFAULT_POP,
         MVM_REPR_DEFAULT_UNSHIFT,
         shift,
         MVM_REPR_DEFAULT_SPLICE,
+        MVM_REPR_DEFAULT_AT_POS_MULTIDIM,
+        MVM_REPR_DEFAULT_BIND_POS_MULTIDIM,
+        MVM_REPR_DEFAULT_DIMENSIONS,
+        MVM_REPR_DEFAULT_SET_DIMENSIONS,
         MVM_REPR_DEFAULT_GET_ELEM_STORAGE_SPEC
     },    /* pos_funcs */
     MVM_REPR_DEFAULT_ASS_FUNCS,
@@ -231,6 +234,8 @@ static const MVMREPROps this_repr = {
     "ConcBlockingQueue", /* name */
     MVM_REPR_ID_ConcBlockingQueue,
     0, /* refs_frames */
+    NULL, /* unmanaged_size */
+    NULL, /* describe_refs */
 };
 
 /* Polls a queue for a value, returning NULL if none is available. */
