@@ -335,7 +335,7 @@ static void set_size_internal(MVMThreadContext *tc, MVMArrayBody *body, MVMint64
 
     /* fill out any unused slots with NULL pointers or zero values */
     body->slots.any = slots;
-    elems = zero_slots(tc, body, elems, ssize, repr_data->slot_type);
+    zero_slots(tc, body, elems, ssize, repr_data->slot_type);
 
     body->ssize = ssize;
 }
@@ -887,6 +887,12 @@ static void set_dimensions(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
 static MVMStorageSpec get_elem_storage_spec(MVMThreadContext *tc, MVMSTable *st) {
     MVMArrayREPRData *repr_data = (MVMArrayREPRData *)st->REPR_data;
     MVMStorageSpec spec;
+
+    /* initialise storage spec to default values */
+    spec.bits            = 0;
+    spec.align           = 0;
+    spec.is_unsigned     = 0;
+
     switch (repr_data->slot_type) {
         case MVM_ARRAY_STR:
             spec.inlineable      = MVM_STORAGE_SPEC_INLINED;
@@ -1188,6 +1194,44 @@ static void spesh(MVMThreadContext *tc, MVMSTable *st, MVMSpeshGraph *g, MVMSpes
     }
 }
 
+/* Calculates the non-GC-managed memory we hold on to. */
+static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data) {
+    MVMArrayREPRData *repr_data = (MVMArrayREPRData *) st->REPR_data;
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+    return body->ssize * repr_data->elem_size;
+}
+
+static void describe_refs (MVMThreadContext *tc, MVMHeapSnapshotState *ss, MVMSTable *st, void *data) {
+    MVMArrayREPRData *repr_data = (MVMArrayREPRData *) st->REPR_data;
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+    MVMuint64         elems     = body->elems;
+    MVMuint64         start     = body->start;
+    MVMuint64         i         = 0;
+
+    switch (repr_data->slot_type) {
+        case MVM_ARRAY_OBJ: {
+            MVMObject **slots = body->slots.o;
+            slots += start;
+            while (i < elems) {
+                MVM_profile_heap_add_collectable_rel_idx(tc, ss,
+                    (MVMCollectable *)slots[i], i);
+                i++;
+            }
+            break;
+        }
+        case MVM_ARRAY_STR: {
+            MVMString **slots = body->slots.s;
+            slots += start;
+            while (i < elems) {
+                MVM_profile_heap_add_collectable_rel_idx(tc, ss,
+                    (MVMCollectable *)slots[i], i);
+                i++;
+            }
+            break;
+        }
+    }
+}
+
 /* Initializes the representation. */
 const MVMREPROps * MVMArray_initialize(MVMThreadContext *tc) {
     return &this_repr;
@@ -1234,4 +1278,6 @@ static const MVMREPROps this_repr = {
     "VMArray", /* name */
     MVM_REPR_ID_MVMArray,
     0, /* refs_frames */
+    unmanaged_size,
+    describe_refs,
 };
