@@ -26,9 +26,10 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
     MVMHashBody *src_body  = (MVMHashBody *)src;
     MVMHashBody *dest_body = (MVMHashBody *)dest;
     MVMHashEntry *current, *tmp;
+    unsigned bucket_tmp;
 
     /* NOTE: if we really wanted to, we could avoid rehashing... */
-    HASH_ITER(hash_handle, src_body->hash_head, current, tmp) {
+    HASH_ITER(hash_handle, src_body->hash_head, current, tmp, bucket_tmp) {
         size_t klen;
         void *kdata;
         MVMHashEntry *new_entry = MVM_fixed_size_alloc(tc, tc->instance->fsa,
@@ -45,8 +46,9 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
 static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorklist *worklist) {
     MVMHashBody *body = (MVMHashBody *)data;
     MVMHashEntry *current, *tmp;
+    unsigned bucket_tmp;
 
-    HASH_ITER(hash_handle, body->hash_head, current, tmp) {
+    HASH_ITER(hash_handle, body->hash_head, current, tmp, bucket_tmp) {
         MVM_gc_worklist_add(tc, worklist, &current->key);
         MVM_gc_worklist_add(tc, worklist, &current->value);
     }
@@ -55,7 +57,16 @@ static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorkli
 /* Called by the VM in order to free memory associated with this object. */
 static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
     MVMHash *h = (MVMHash *)obj;
-    MVM_HASH_DESTROY_FSA(hash_handle, MVMHashEntry, h->body.hash_head);
+    MVMHashEntry *current, *tmp;
+    unsigned bucket_tmp;
+    HASH_ITER(hash_handle, h->body.hash_head, current, tmp, bucket_tmp) {
+        if (current != h->body.hash_head)
+            MVM_fixed_size_free(tc, tc->instance->fsa, sizeof(MVMHashEntry), current);
+    }
+    tmp = h->body.hash_head;
+    HASH_CLEAR(hash_handle, h->body.hash_head);
+    if (tmp)
+        MVM_fixed_size_free(tc, tc->instance->fsa, sizeof(MVMHashEntry), tmp);
 }
 
 static void at_key(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *key, MVMRegister *result, MVMuint16 kind) {
@@ -184,6 +195,12 @@ static void spesh(MVMThreadContext *tc, MVMSTable *st, MVMSpeshGraph *g, MVMSpes
     }
 }
 
+static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data) {
+    MVMHashBody *body = (MVMHashBody *)data;
+
+    return sizeof(MVMHashEntry) * HASH_CNT(hash_handle, body->hash_head);
+}
+
 /* Initializes the representation. */
 const MVMREPROps * MVMHash_initialize(MVMThreadContext *tc) {
     return &this_repr;
@@ -222,4 +239,6 @@ static const MVMREPROps this_repr = {
     "VMHash", /* name */
     MVM_REPR_ID_MVMHash,
     0, /* refs_frames */
+    unmanaged_size, /* unmanaged_size */
+    NULL, /* describe_refs */
 };
