@@ -39,9 +39,9 @@ static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
     MVMMultiCache *mc = (MVMMultiCache *)obj;
     MVMint64 i;
     for (i = 0; i < MVM_MULTICACHE_MAX_ARITY; i++) {
-        MVM_checked_free_null(mc->body.arity_caches[i].type_ids);
-        MVM_checked_free_null(mc->body.arity_caches[i].named_ok);
-        MVM_checked_free_null(mc->body.arity_caches[i].results);
+        MVM_free(mc->body.arity_caches[i].type_ids);
+        MVM_free(mc->body.arity_caches[i].named_ok);
+        MVM_free(mc->body.arity_caches[i].results);
     }
 }
 
@@ -64,6 +64,23 @@ static void compose(MVMThreadContext *tc, MVMSTable *st, MVMObject *info) {
     /* Nothing to do for this REPR. */
 }
 
+/* Calculates the non-GC-managed memory we hold on to. */
+static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data) {
+    MVMMultiCacheBody *body = (MVMMultiCacheBody *)data;
+    MVMuint8 arity;
+    MVMuint64 size = 0;
+
+    for (arity = 0; arity < MVM_MULTICACHE_MAX_ARITY; arity++) {
+        if (body->arity_caches[arity].num_entries != 0) {
+            size += (arity + 1) * sizeof(MVMuint64  ) * MVM_MULTICACHE_MAX_ENTRIES; /* type_ids */
+            size +=               sizeof(MVMuint8   ) * MVM_MULTICACHE_MAX_ENTRIES; /* named_os */
+            size +=               sizeof(MVMObject *) * MVM_MULTICACHE_MAX_ENTRIES; /* results  */
+        }
+    }
+
+    return size;
+}
+
 /* Initializes the representation. */
 const MVMREPROps * MVMMultiCache_initialize(MVMThreadContext *tc) {
     return &this_repr;
@@ -96,6 +113,8 @@ static const MVMREPROps this_repr = {
     "MVMMultiCache", /* name */
     MVM_REPR_ID_MVMMultiCache,
     0, /* refs_frames */
+    unmanaged_size, /* unmanaged_size */
+    NULL, /* describe_refs */
 };
 
 MVMObject * MVM_multi_cache_add(MVMThreadContext *tc, MVMObject *cache_obj, MVMObject *capture, MVMObject *result) {
@@ -163,27 +182,33 @@ MVMObject * MVM_multi_cache_add(MVMThreadContext *tc, MVMObject *cache_obj, MVMO
         if (arg_type == MVM_CALLSITE_ARG_OBJ) {
             MVMObject *arg = MVM_args_get_pos_obj(tc, apc, i, 1).arg.o;
             if (arg) {
+                MVMuint8 rwness = 0;
                 MVMContainerSpec const *contspec = STABLE(arg)->container_spec;
                 if (contspec && IS_CONCRETE(arg)) {
                     if (contspec->fetch_never_invokes) {
                         if (REPR(arg)->ID != MVM_REPR_ID_NativeRef) {
                             MVMRegister r;
+                            if (contspec->can_store(tc, arg))
+                                rwness = 2;
                             contspec->fetch(tc, arg, &r);
                             arg = r.o;
                         }
+                        else {
+                            rwness = 2; /* Native refs are always writable. */
+                        }
                     }
                     else {
                         goto DONE;
                     }
                 }
-                arg_tup[i] = STABLE(arg)->type_cache_id | (IS_CONCRETE(arg) ? 1 : 0);
+                arg_tup[i] = STABLE(arg)->type_cache_id | (IS_CONCRETE(arg) ? 1 : 0) | rwness;
             }
             else {
                 goto DONE;
             }
         }
         else {
-            arg_tup[i] = (arg_type << 1) | 1;
+            arg_tup[i] = (arg_type << 2) | 1;
         }
     }
 
@@ -257,27 +282,33 @@ MVMObject * MVM_multi_cache_find(MVMThreadContext *tc, MVMObject *cache_obj, MVM
         if (arg_type == MVM_CALLSITE_ARG_OBJ) {
             MVMObject *arg = MVM_args_get_pos_obj(tc, apc, i, 1).arg.o;
             if (arg) {
+                MVMuint8 rwness = 0;
                 MVMContainerSpec const *contspec = STABLE(arg)->container_spec;
                 if (contspec && IS_CONCRETE(arg)) {
                     if (contspec->fetch_never_invokes) {
                         if (REPR(arg)->ID != MVM_REPR_ID_NativeRef) {
                             MVMRegister r;
+                            if (contspec->can_store(tc, arg))
+                                rwness = 2;
                             contspec->fetch(tc, arg, &r);
                             arg = r.o;
                         }
+                        else {
+                            rwness = 2; /* Native refs are always writable. */
+                        }
                     }
                     else {
                         return NULL;
                     }
                 }
-                arg_tup[i] = STABLE(arg)->type_cache_id | (IS_CONCRETE(arg) ? 1 : 0);
+                arg_tup[i] = STABLE(arg)->type_cache_id | (IS_CONCRETE(arg) ? 1 : 0) | rwness;
             }
             else {
                 return NULL;
             }
         }
         else {
-            arg_tup[i] = (arg_type << 1) | 1;
+            arg_tup[i] = (arg_type << 2) | 1;
         }
     }
 
@@ -337,27 +368,33 @@ MVMObject * MVM_multi_cache_find_callsite_args(MVMThreadContext *tc, MVMObject *
         if (arg_type == MVM_CALLSITE_ARG_OBJ) {
             MVMObject *arg = args[i].o;
             if (arg) {
+                MVMuint8 rwness = 0;
                 MVMContainerSpec const *contspec = STABLE(arg)->container_spec;
-                if (contspec && IS_CONCRETE(arg)) {
+                if (contspec && IS_CONCRETE(arg)) {    
                     if (contspec->fetch_never_invokes) {
                         if (REPR(arg)->ID != MVM_REPR_ID_NativeRef) {
                             MVMRegister r;
+                            if (contspec->can_store(tc, arg))
+                                rwness = 2;
                             contspec->fetch(tc, arg, &r);
                             arg = r.o;
                         }
+                        else {
+                            rwness = 2; /* Native refs are always writable. */
+                        }
                     }
                     else {
                         return NULL;
                     }
                 }
-                arg_tup[i] = STABLE(arg)->type_cache_id | (IS_CONCRETE(arg) ? 1 : 0);
+                arg_tup[i] = STABLE(arg)->type_cache_id | (IS_CONCRETE(arg) ? 1 : 0) | rwness;
             }
             else {
                 return NULL;
             }
         }
         else {
-            arg_tup[i] = (arg_type << 1) | 1;
+            arg_tup[i] = (arg_type << 2) | 1;
         }
     }
 
@@ -426,7 +463,7 @@ MVMObject * MVM_multi_cache_find_spesh(MVMThreadContext *tc, MVMObject *cache_ob
 
                 /* If it's a container, must know what's inside it. Otherwise,
                  * we're already good on type info. */
-                if ((facts->flags & MVM_SPESH_FACT_CONCRETE)&& STABLE(facts->type)->container_spec) {
+                if ((facts->flags & MVM_SPESH_FACT_CONCRETE) && STABLE(facts->type)->container_spec) {
                     /* Again, need to know type and concreteness. */
                     if (!(facts->flags & MVM_SPESH_FACT_KNOWN_DECONT_TYPE))
                         return NULL;
@@ -434,6 +471,8 @@ MVMObject * MVM_multi_cache_find_spesh(MVMThreadContext *tc, MVMObject *cache_ob
                         return NULL;
                     arg_tup[i] = STABLE(facts->decont_type)->type_cache_id |
                         ((facts->flags & MVM_SPESH_FACT_DECONT_CONCRETE) ? 1 : 0);
+                    if (facts->flags & MVM_SPESH_FACT_RW_CONT)
+                        arg_tup[i] |= 2;
                 }
                 else {
                     arg_tup[i] = STABLE(facts->type)->type_cache_id |
@@ -445,7 +484,7 @@ MVMObject * MVM_multi_cache_find_spesh(MVMThreadContext *tc, MVMObject *cache_ob
             }
         }
         else {
-            arg_tup[i] = (arg_type << 1) | 1;
+            arg_tup[i] = (arg_type << 2) | 1;
         }
     }
 
