@@ -188,4 +188,6 @@ static const MVMREPROps this_repr = {
     "P6num", /* name */
     MVM_REPR_ID_P6num,
     0, /* refs_frames */
+    NULL, /* unmanaged_size */
+    NULL, /* describe_refs */
 };
