@@ -111,4 +111,6 @@ static const MVMREPROps this_repr = {
     "P6str", /* name */
     MVM_REPR_ID_P6str,
     0, /* refs_frames */
+    NULL, /* unmanaged_size */
+    NULL, /* describe_refs */
 };
