@@ -41,7 +41,7 @@ static void add_in_tray_to_worklist(MVMThreadContext *tc, MVMGCWorklist *worklis
  * building up a huge worklist. */
 void MVM_gc_collect(MVMThreadContext *tc, MVMuint8 what_to_do, MVMuint8 gen) {
     /* Create a GC worklist. */
-    MVMGCWorklist *worklist = MVM_gc_worklist_create(tc, gen != MVMGCGenerations_Nursery);
+    MVMGCWorklist *worklist = MVM_gc_worklist_create(tc, gen != MVMGCGenerations_Nursery, 0);
 
     /* Initialize work passing data structure. */
     WorkToPass wtp;
@@ -78,23 +78,19 @@ void MVM_gc_collect(MVMThreadContext *tc, MVMuint8 what_to_do, MVMuint8 gen) {
         tc->nursery_alloc       = tospace;
         tc->nursery_alloc_limit = (char *)tc->nursery_alloc + MVM_NURSERY_SIZE;
 
-        MVM_gc_worklist_add(tc, worklist, &tc->thread_obj);
-        GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from thread_obj\n", worklist->items);
-        process_worklist(tc, worklist, &wtp, gen);
-
         /* Add permanent roots and process them; only one thread will do
         * this, since they are instance-wide. */
         if (what_to_do != MVMGCWhatToDo_NoInstance) {
-            MVM_gc_root_add_permanents_to_worklist(tc, worklist);
+            MVM_gc_root_add_permanents_to_worklist(tc, worklist, NULL);
             GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from instance permanents\n", worklist->items);
             process_worklist(tc, worklist, &wtp, gen);
-            MVM_gc_root_add_instance_roots_to_worklist(tc, worklist);
+            MVM_gc_root_add_instance_roots_to_worklist(tc, worklist, NULL);
             GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from instance roots\n", worklist->items);
             process_worklist(tc, worklist, &wtp, gen);
         }
 
         /* Add per-thread state to worklist and process it. */
-        MVM_gc_root_add_tc_roots_to_worklist(tc, worklist);
+        MVM_gc_root_add_tc_roots_to_worklist(tc, worklist, NULL);
         GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from TC objects\n", worklist->items);
         process_worklist(tc, worklist, &wtp, gen);
 
@@ -104,7 +100,7 @@ void MVM_gc_collect(MVMThreadContext *tc, MVMuint8 what_to_do, MVMuint8 gen) {
         process_worklist(tc, worklist, &wtp, gen);
 
         /* Add temporary roots and process them (these are per-thread). */
-        MVM_gc_root_add_temps_to_worklist(tc, worklist);
+        MVM_gc_root_add_temps_to_worklist(tc, worklist, NULL);
         GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from thread temps\n", worklist->items);
         process_worklist(tc, worklist, &wtp, gen);
 
@@ -119,13 +115,6 @@ void MVM_gc_collect(MVMThreadContext *tc, MVMuint8 what_to_do, MVMuint8 gen) {
             process_worklist(tc, worklist, &wtp, gen);
         }
 
-        /* Find roots in frames and process them. */
-        if (tc->cur_frame) {
-            MVM_gc_worklist_add_frame(tc, worklist, tc->cur_frame);
-            GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from cur_frame \n", worklist->items);
-            process_worklist(tc, worklist, &wtp, gen);
-        }
-
         /* Process anything in the in-tray. */
         add_in_tray_to_worklist(tc, worklist);
         GCDEBUG_LOG(tc, MVM_GC_DEBUG_COLLECT, "Thread %d run %d : processing %d items from in tray \n", worklist->items);
@@ -246,7 +235,9 @@ static void process_worklist(MVMThreadContext *tc, MVMGCWorklist *worklist, Work
                     ? MVM_gc_object_id_use_allocation(tc, item)
                     : MVM_gc_gen2_allocate(gen2, item->size);
 
-                /* Add on to the promoted amount (used by profiler). */
+                /* Add on to the promoted amount (used both to decide when to do
+                 * the next full collection, as well as for profiling). Note we
+                 * add unmanaged size on for objects below. */
                 tc->gc_promoted_bytes += item->size;
 
                 /* Copy the object to the second generation and mark it as
@@ -258,11 +249,14 @@ static void process_worklist(MVMThreadContext *tc, MVMGCWorklist *worklist, Work
                 new_addr->flags |= MVM_CF_SECOND_GEN;
 
                 /* If it references frames or static frames, we need to keep
-                 * on visiting it. */
+                 * on visiting it. Also add on object's unmanaged size. */
                 if (!(new_addr->flags & (MVM_CF_TYPE_OBJECT | MVM_CF_STABLE))) {
                     MVMObject *new_obj_addr = (MVMObject *)new_addr;
                     if (REPR(new_obj_addr)->refs_frames)
                         MVM_gc_root_gen2_add(tc, (MVMCollectable *)new_obj_addr);
+                    if (REPR(new_obj_addr)->unmanaged_size)
+                        tc->gc_promoted_bytes += REPR(new_obj_addr)->unmanaged_size(tc,
+                            STABLE(new_obj_addr), OBJECT_BODY(new_obj_addr));
                 }
 
                 /* If we're going to sweep the second generation, also need
